//
// Created by zx on 2020/1/3.
//

#include "System_Manager.h"
#include <glog/logging.h>
#include <google/protobuf/io/zero_copy_stream_impl.h>
#include <google/protobuf/text_format.h>
#include <fcntl.h>
#include <livox_sdk.h>
using google::protobuf::io::FileInputStream;
using google::protobuf::io::FileOutputStream;
using google::protobuf::io::ZeroCopyInputStream;
using google::protobuf::io::CodedInputStream;
using google::protobuf::io::ZeroCopyOutputStream;
using google::protobuf::io::CodedOutputStream;
using google::protobuf::Message;

System_Manager::System_Manager()
{
    m_laser_vector.clear();
    mp_terminal=NULL;
    mp_plc=NULL;
    mp_locater=NULL;

}
System_Manager::~System_Manager()
{
    //第一步, 清除流程,保证不会再调用plc,然后最先析构plc,防止回调崩溃
    if(mp_terminal!=NULL)
    {
        mp_terminal->force_stop_command();
    }

    //析构plc
    if(mp_plc!=NULL)
    {
        delete mp_plc;
        mp_plc=NULL;
    }
    //第二步析构terminor
    if(mp_terminal!=NULL)
    {
        delete mp_terminal;
        mp_terminal=NULL;
    }

    //析构locater
    if(mp_locater!=NULL)
    {
        delete mp_locater;
        mp_locater=NULL;
    }
    //析构laser
    for(int i=0;i<m_laser_vector.size();++i)
    {
        if(m_laser_vector[i]!=NULL)
        {
            delete m_laser_vector[i];
            m_laser_vector[i]=NULL;
        }
    }
    m_laser_vector.clear();



}
//初始化各个模块,包括雷达,plc,locater,terminor
//1,读取运行环境下setting目录下的laser.prototxt,plc.prototxt,locater.prototxt,terminal.prototxt
//2,根据配置创建雷达,plc,locater,terminal
Error_manager System_Manager::init()
{
    MeasureTopicPublisher::GetInstance();
    Error_manager code;
    //读laser配置
    Laser_proto::Laser_parameter_all laser_parameters;
    if(!read_proto_param("./setting/laser.prototxt",laser_parameters))
    {
        return Error_manager(SYSTEM_READ_PARAMETER_ERROR,NORMAL,"read laser parameter failed");
    }

    //读取terminor配置
    Terminal::Terminor_parameter_all terminor_parameter_all;
    if(!read_proto_param("./setting/terminal.prototxt",terminor_parameter_all))
    {
        return Error_manager(SYSTEM_READ_PARAMETER_ERROR,NORMAL,"read terminal parameter failed");
    }
    m_terminal_parameter_all=terminor_parameter_all;
    //第二步,根据配置,创建各个模块
    //创建雷达
    int laser_cout=laser_parameters.laser_parameters_size();
    m_laser_vector.resize(laser_cout);
    for(int i=0;i<laser_parameters.laser_parameters_size();++i)
    {
        m_laser_vector[i]=LaserRegistory::CreateLaser(laser_parameters.laser_parameters(i).type(),i,
                                                      laser_parameters.laser_parameters(i));
        if(m_laser_vector[i]!=NULL)
        {
            if(m_laser_vector[i]->connect_laser()!=SUCCESS)
            {
                char description[255]={0};
                sprintf(description,"Laser %d connect failed...",i);
                return Error_manager(LASER_CONNECT_FAILED,NORMAL,description);
            }
        }
    }
    //查询是否有livox雷达,初始化库
    for (int i = 0; i < laser_parameters.laser_parameters_size(); ++i)
    {
        std::string type = laser_parameters.laser_parameters(i).type();
        if (type.find("Livox") != type.npos)
        {
            if (Start() == false)
            {
                Uninit();
                return Error_manager(LASER_LIVOX_SKD_INIT_FAILED,NORMAL,"Livox laser init failed...");
            }
            break;
        }
    }
    if(code!=SUCCESS)
    {
        return code;
    }

    //创建terminor
    mp_terminal=new Terminor_Command_Executor(terminor_parameter_all.terminor_parameters(0));
    mp_terminal->set_save_root_path(terminor_parameter_all.save_root_path());

    return code;

}

//plc指令回调函数,当plc收到测量指令后,调用此回调函数
//terminal_id:收到指令的终端编号
//owner:this指针
Error_manager System_Manager::start_DJ_scanning()
{
    Error_manager code;

    //第一步判断输入参数是否合法

    System_Manager* system_manager=this;

    //第二步,根据terminal_id,启动对应terminaor执行指令流程,

    if(system_manager->m_laser_vector.size()==0)
    {
        return Error_manager(SYSTEM_INPUT_TERMINOR_NO_LASERS,NORMAL,"laser parameter has no laser");
    }

    code=system_manager->mp_terminal->execute_command(system_manager->m_laser_vector,10);
    switch(code.get_error_code())
    {
        case SUCCESS:break;
        case TERMINOR_CREATE_WORKING_THREAD_FAILED:
        {
            LOG(ERROR)<<code.to_string();
            break;
        }
        default:
        {
            LOG(WARNING)<<code.to_string();
            break;
        }
    }
    return code;

}

//读取protobuf 配置文件
//file:文件
//parameter:要读取的配置
bool System_Manager::read_proto_param(std::string file, ::google::protobuf::Message& parameter)
{
    int fd = open(file.c_str(), O_RDONLY);
    if (fd == -1) return false;
    FileInputStream* input = new FileInputStream(fd);
    bool success = google::protobuf::TextFormat::Parse(input, &parameter);
    delete input;
    close(fd);
    return success;
}